/* Copyright (c) 2006, 2010, Oracle and/or its affiliates. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1335  USA */

#ifndef SQL_HELP_INCLUDED
#define SQL_HELP_INCLUDED

class THD;


/*
  Function prototypes
*/

bool mysqld_help (THD *thd, const char *text);

bool mysqld_help_prepare(THD *thd, const char *text, List<Item> *fields);

#endif /* SQL_HELP_INCLUDED */
